#pragma once
#include <linalg/type.hpp>
#include <optmz/minmz.hpp>
// No assumption on whether knots/x is sorted
Eigen::ArrayXi lower_bound_index(const dynvec_t& x, const dynvec_t& knots);
dynmat_t natural_f(dynvec_t knots);
dynmat_t get_free_crs_dmat(dynvec_t x, dynvec_t knots, dynmat_t f, int deriv_order);

struct spline_base_t {
  Eigen::ArrayXd ajm, ajp, cjm, cjp;
  Eigen::ArrayXi j;
};
spline_base_t compute_base_functions(const dynvec_t& x, const dynvec_t& knots, int deriv_order);

/* a default cost/dcost is choosed.
 * c-style api
 * */
struct volfit_iarray_t {
  // all array are allocated by constructor, user shall then fill in the correct input values.
  explicit volfit_iarray_t(int num_fitdata, int num_knots);
  ~volfit_iarray_t();
  int num_fitdata;
  int num_knots;
  double* fitdata_x;
  double* fitdata_values;
  double* fitdata_weights;
  double* knots;
  double* x0;
};

/* a default cost/dcost for cubic spline fitting problem will be constructed
 */
struct cubic_spline_input_t : public constraints_t {
  dynvec_t knots;
  dynvec_t x0; // knots initial value
  dynvec_t fitdata_x;
  dynvec_t fitdata_values;
  dynvec_t fitdata_weights;
  int maxiter;
  double ftol;
  double loss_scale;
  bounds_t bounds;
};

struct cubic_spline_ws_t_impl;
/*
 * cubic spline fitting, using slsqp
 * */

struct cubic_spline_ws_t {
  struct iter_vars_t {
    dynvec_t coef;
    dynvec_t err;
    double ret;
  };

  cubic_spline_ws_t();
  ~cubic_spline_ws_t();

  void fit(cubic_spline_input_t);

  const slsqp_ws_t& slsqp() const;

  bool success() const {
    return slsqp_.success();
  }

  dynvec_t x() const {
    return slsqp_.x();
  }

  double fx() const {
    return slsqp_.fx();
  }

  dynvec_t g() const {
    return slsqp_.g();
  }

  dynvec_t lb() const {
    return slsqp_.lb();
  }

  dynvec_t ub() const {
    return slsqp_.ub();
  }

  int numiter() const {
    return slsqp_.numiter();
  }
  int mode() const {
    return slsqp_.mode();
  }

  const char* modestr() const {
    return slsqp_.modestr();
  }

  size_t num_cost_eval() const;
  // starting from 0, nullptr if out of range
  const iter_vars_t* cost_eval_data(size_t iter) const;
  dynmat_t spline_basis() const;
private:
  slsqp_ws_t slsqp_;
  std::unique_ptr<cubic_spline_ws_t_impl> impl_;
};
