#include <pybind11/detail/common.h>
#include <pybind11/pybind11.h>
#include <pybind11/functional.h>
#include <pybind11/stl.h>
#include <pybind11/eigen.h>
#include <optmz/minmz.hpp>
#include <fit/crs.hpp>
namespace py = pybind11;
PYBIND11_MODULE(pyfit, m) {
  py::class_<bounds_t>(m, "Bounds")
    .def(py::init<>())
    .def_readwrite("l", &bounds_t::l)
    .def_readwrite("u", &bounds_t::u)
    .def_readwrite("lv", &bounds_t::lv)
    .def_readwrite("uv", &bounds_t::uv)
  ;
  py::class_<minmz_i_t>(m, "MinmzRawInput")
    .def(py::init<>())
    .def_readwrite("func", &minmz_i_t::func)
    .def_readwrite("jac", &minmz_i_t::jac)
    .def_readwrite("x0", &minmz_i_t::x0)
    .def_readwrite("maxiter", &minmz_i_t::maxiter)
    .def_readwrite("ftol", &minmz_i_t::ftol)
    .def_readwrite("bounds", &minmz_i_t::bounds)
    .def("add_eq_con", &minmz_i_t::add_eq_con)
    .def("add_ieq_con", &minmz_i_t::add_ieq_con)
    .def("num_cons", &minmz_i_t::num_cons)
    .def("sum_eq_cons_rows", &minmz_i_t::sum_eq_cons_rows)
    .def("sum_ieq_cons_rows", &minmz_i_t::sum_ieq_cons_rows)
  ;
  py::class_<slsqp_ws_t>(m, "SLSQPWorkspace")
    .def(py::init<>())
    .def("minimize", &slsqp_ws_t::minimize)
    .def("success", &slsqp_ws_t::success)
    .def("x", &slsqp_ws_t::x)
    .def("fx", &slsqp_ws_t::fx)
    .def("g", &slsqp_ws_t::g)
    .def("lb", &slsqp_ws_t::lb)
    .def("ub", &slsqp_ws_t::ub)
    .def("numiter", &slsqp_ws_t::numiter)
    .def("mode", &slsqp_ws_t::mode)
    .def("modestr", &slsqp_ws_t::modestr)
  ;
  py::class_<cubic_spline_input_t>(m, "CubicSplineFitInput")
    .def(py::init<>())
    .def_readwrite("knots", &cubic_spline_input_t::knots)
    .def_readwrite("x0", &cubic_spline_input_t::x0)
    .def_readwrite("fitdata_x", &cubic_spline_input_t::fitdata_x)
    .def_readwrite("fitdata_values", &cubic_spline_input_t::fitdata_values)
    .def_readwrite("fitdata_weights", &cubic_spline_input_t::fitdata_weights)
    .def_readwrite("maxiter", &cubic_spline_input_t::maxiter)
    .def_readwrite("ftol", &cubic_spline_input_t::ftol)
    .def_readwrite("loss_scale", &cubic_spline_input_t::loss_scale)
    .def_readwrite("bounds", &cubic_spline_input_t::bounds)
    .def("add_eq_con", &minmz_i_t::add_eq_con)
    .def("add_eq_con", &cubic_spline_input_t::add_eq_con)
    .def("add_ieq_con", &cubic_spline_input_t::add_ieq_con)
    .def("num_cons", &cubic_spline_input_t::num_cons)
    .def("sum_eq_cons_rows", &cubic_spline_input_t::sum_eq_cons_rows)
    .def("sum_ieq_cons_rows", &cubic_spline_input_t::sum_ieq_cons_rows)
  ;
  py::class_<cubic_spline_ws_t::iter_vars_t>(m, "CubicSplineCostVars")
    .def_readonly("coef", &cubic_spline_ws_t::iter_vars_t::coef)
    .def_readonly("err", &cubic_spline_ws_t::iter_vars_t::err)
    .def_readonly("ret", &cubic_spline_ws_t::iter_vars_t::ret)
  ;
  py::class_<cubic_spline_ws_t>(m, "CubicSplineWorkspace")
    .def(py::init<>())
    .def("fit", &cubic_spline_ws_t::fit)
    .def("success", &cubic_spline_ws_t::success)
    .def("x", &cubic_spline_ws_t::x)
    .def("fx", &cubic_spline_ws_t::fx)
    .def("g", &cubic_spline_ws_t::g)
    .def("lb", &cubic_spline_ws_t::lb)
    .def("ub", &cubic_spline_ws_t::ub)
    .def("numiter", &cubic_spline_ws_t::numiter)
    .def("mode", &cubic_spline_ws_t::mode)
    .def("modestr", &cubic_spline_ws_t::modestr)
    .def("num_cost_eval", &cubic_spline_ws_t::num_cost_eval)
    .def("cost_eval_data", &cubic_spline_ws_t::cost_eval_data, py::return_value_policy::reference_internal)
    .def("spline_basis", &cubic_spline_ws_t::spline_basis)
  ;
}
